import re

class Case:

    def _CamelSep(string:str, sep:str):
        """ Don't use it!"""
        _list = string.split(sep)
        first = _list[0].lower()
        last = []
        for i in _list[1:]:
            last.append(i.title())
        last = "".join(last)
        result = str(first + last)
        return result

    def _OthersSep(string: str, sep:str, sep1:str, case:str):
        """ Don't use it!"""
        _list = string.split(sep)
        last = []
        for i in _list:
            if case == "lower":
                last.append(i.lower())
            elif case == "upper":
                last.append(i.upper())
            elif case == "title":
                last.append(i.title())
        result = sep1.join(last)
        return result

    def _Error():
        """ Don't use it!"""
        #if feedback:
            #print(f"Was returned a {case1} case, because the string has no differentiator")
        raise ValueError("case is wrong, choose between: 'lower', 'upper' or 'title'")


    def to_camel(string: str):
        """toCamel
        string: str, case1: str = "lower"

        Returns a "camelCase"

        + string is the input - It can be in a Pascal, a Sentence, Snake or Kebab case.
        + case1 defines the output case of a string without differentiador, (" ", "-", "_")
        + case1 options: "lower", "title", "upper"

        """
        string = string.strip()
        
        # For Sentences:
        if len(string.split(" ")) > 1:
            return Case._CamelSep(string, " ")
        
        # For Snakes:
        elif len(string.split("_")) > 1:
            return Case._CamelSep(string, "_")
            
        # For Kebab:
        elif len(string.split("-")) > 1:
            return Case._CamelSep(string, "-")
    
        # For Uppers, Titles and Lowers:
        elif string.istitle() or string.isupper() or string.islower():
            Case._Error()
        # Errors:
        elif string.isdecimal() or string.isdigit() or string.isnumeric():
            raise ValueError("It's a number")
        elif string == "":
            raise ValueError("It's a white space")

        # For Pascal:
        else:
            _list = re.findall('[A-Z][^A-Z]*', string)
            first = _list[0].lower()
            last = _list[1:]
            last = "".join(last)
            pascal = first + last
            return pascal

            

    def to_snake(string: str, case: str = "lower"):
        """toSnake
        string: str, case: str = "lower", case1: str = "lower"

        Returns "snake_case"

        + string is the input - It can be in a Pascal, a Sentence, Snake or Kebab case.
        + case defines the output case of a string with differentiador, (" ", "-", "_")
        + case1 defines the output case of a string without differentiador, (" ", "-", "_")
        + case/case1 options: "lower", "title", "upper"
        """
        string = string.strip()
        sep1 = "_"
        case = case
        
        # For Sentences:
        if len(string.split(" ")) > 1:
            return Case._OthersSep(string, " ", sep1=sep1, case=case)
        
        # For Kebab:
        elif len(string.split("-")) > 1:
            return Case._OthersSep(string, "-", sep1=sep1, case=case)
    
        # For Uppers, Titles and Lowers:
        elif string.istitle() or string.isupper() or string.islower():
            Case._Error()
            
        # Errors:
        elif string.isdecimal() or string.isdigit() or string.isnumeric():
            raise ValueError("It's a number")
        elif string == "":
            raise ValueError("It's a white space")

        # For Pascal/Camel:
        else:
            last = re.findall('[A-Z][^A-Z]*', string)
            for i in re.finditer('[A-Z][^A-Z]*', string):
                index = i.span()[0]
                break
            if index == 0:
                first = ""
                l = []
                for i in last:
                    l.append(i.lower())
                pascal = str("_".join(l))
                return pascal
            else:
                first = string[:index]
                l = []
                for i in last:
                    l.append(i.lower())
                r = str("_".join(l))
                camel = first + "_" + r
                return camel
            


    def to_kebab(string: str, case: str = "lower"):
        """toKebab
        string: str, case: str = "lower", case1: str = "lower"

        Returns "kebab-case"

        + string is the input - It can be in a Pascal, a Sentence, Snake or Kebab case.
        + case defines the output case of a string with differentiador, (" ", "-", "_")
        + case1 defines the output case of a string without differentiador, (" ", "-", "_")
        + case/case1 options: "lower", "title", "upper"
        """
        string = string.strip()
        sep1 = "-"
        case=case
        
        # For Sentences:
        if len(string.split(" ")) > 1:
            return Case._OthersSep(string, " ", sep1=sep1, case=case)
            
        # For Snake:
        elif len(string.split("_")) > 1:
            return Case._OthersSep(string, "_", sep1=sep1, case=case)
    
        # For Uppers, Titles and Lowers:
        elif string.istitle() or string.isupper() or string.islower():
            Case._Error()
        # Errors:
        elif string.isdecimal() or string.isdigit() or string.isnumeric():
            raise ValueError("It's a number")
        elif string == "":
            raise ValueError("It's a white space")

        # For Pascal/Camel:
        else:
            last = re.findall('[A-Z][^A-Z]*', string)
            for i in re.finditer('[A-Z][^A-Z]*', string):
                index = i.span()[0]
                break
            if index == 0:
                first = ""
                l = []
                for i in last:
                    l.append(i.lower())
                pascal = str("-".join(l))
                return pascal
            else:
                first = string[:index]
                l = []
                for i in last:
                    l.append(i.lower())
                r = str("-".join(l))
                camel = first + "-" + r
                return camel



    def to_pascal(string: str):
        """toPascal
        string: str, case: str = "lower", case1: str = "lower"

        Returns "PascalCase"

        + string is the input - It can be in a Pascal, a Sentence, Snake or Kebab case.
        + case1 defines the output case of a string without differentiador, (" ", "-", "_")
        + case1 options: "lower", "title", "upper"
        """
        string = string.strip()
        sep1 = ""
        case= "title"
        
        # For Sentences:
        if len(string.split(" ")) > 1:
            return Case._OthersSep(string, " ", sep1=sep1, case=case)
        
        # For Snakes:
        elif len(string.split("_")) > 1:
            return Case._OthersSep(string, "_", sep1=sep1, case=case)
        
        # For Kebab:
        elif len(string.split("-")) > 1:
            return Case._OthersSep(string, "-", sep1=sep1, case=case)
    
        # For Uppers, Titles and Lowers:
        elif string.istitle() or string.isupper() or string.islower():
            Case._Error()
        # Errors:
        elif string.isdecimal() or string.isdigit() or string.isnumeric():
            raise ValueError("It's a number")
        elif string == "":
            raise ValueError("It's a white space")

        # For Camel:
        else:
            last = re.findall('[A-Z][^A-Z]*', string)
            for i in re.finditer('[A-Z][^A-Z]*', string):
                index = i.span()[0]
                break
            first = string[:index].title()
            l = []
            for i in last:
                l.append(i.title())
            l = "".join(l)
            camel = first + l
            return camel



    def to_upper_snake(string: str):
        """toUpperSnake
        string: str, case: str = "lower", case1: str = "lower"

        Returns "UPPER_SNAKE_CASE"

        + string is the input - It can be in a Pascal, a Sentence, Snake or Kebab case.
        """
        
        # For Lowers, Uppers and Titles:
        if string.islower() or string.isupper() or string.istitle():
            string = string.strip()
            return string.upper()
        else:
            return Case.toSnake(string).upper()



    def to_sentence(string: str, case: str = "lower"):
        """toSentence
        string: str, case: str = "lower", case1: str = "lower"

        Returns "String sentence"

        + string is the input - It can be in a Pascal, a Sentence, Snake or Kebab case.
        + case defines the output case of a string with differentiador, (" ", "-", "_")
        + case1 defines the output case of a string without differentiador, (" ", "-", "_")
        + case/case1 options: "lower", "title", "upper"
        """
        string = string.strip()
        sep1= " "
        case=case

        if len(string.split(" ")) > 1:
            return Case._OthersSep(string, " ", sep1=sep1, case=case)
        
        # For Snake:
        elif len(string.split("_")) > 1:
            return Case._OthersSep(string, "_", sep1=sep1, case=case)
        
        # For Kebab:
        elif len(string.split("-")) > 1:
            return Case._OthersSep(string, "-", sep1=sep1, case=case)
    
        # For Uppers, Titles and Lowers:
        elif string.istitle() or string.isupper() or string.islower():
            Case._Error()
        # Errors:
        elif string.isdecimal() or string.isdigit() or string.isnumeric():
            raise ValueError("It's a number")
        elif string == "":
            raise ValueError("It's a white space")

        # For Pascal/Camel:
        else:
            last = re.findall('[A-Z][^A-Z]*', string)
            for i in re.finditer('[A-Z][^A-Z]*', string):
                index = i.span()[0]
                break

            if index == 0:
                first = ""
                l = []
                for i in last:
                    if case == "lower":
                        l.append(i.lower())
                    elif case == "upper":
                        l.append(i.upper())
                    elif case == "title":
                        l.append(i.title())
                pascal = str(" ".join(l))
                return pascal
            
            else:
                first = string[:index]
                if case == "lower":
                        first = first.lower()
                elif case == "upper":
                        first = first.upper()
                elif case == "title":
                        first = first.title()
                l = []
                for i in last:
                    if case == "lower":
                        l.append(i.lower())
                    elif case == "upper":
                        l.append(i.upper())
                    elif case == "title":
                        l.append(i.title())
                r = str(" ".join(l))
                camel = first + " " + r
                return camel


"""
toCase

toCase is a Case converter made in python, for peoples who wants simplify this feature. It can convert to and from Camel, Pascal, Snake, Kebab and Strings Sentences.
"""

from .rb_tocase import *

__version__ = "1.3.1"
__author__ = 'RickBarretto'
__credits__ = 'RickBarretto'


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rb_tocase']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'rb-tocase',
    'version': '1.3.2',
    'description': 'RB toCase is a Case converter.',
    'long_description': '<h1 align="center">RB toCase<h1>\n<img alt="Cover" src="./assets/cover.svg">\n<h4 align="center">made by: <a href="https://github.com/RickBarretto/">RickBarreto</a></h4>\n\n## What is it?\n**RB toCase** is a Case converter made in python, for peoples who wants simplify this feature. It can convert to and from Camel, Pascal, Snake, Kebab and Strings Sentences.\n**And, You don\'t need say what is the input type, the code parse it. Just say whats is the output type.**\n\n> Older name was toCase, but when I was publishing on Pipy tocase package already exists. So I changed the name!\n\n## Install\n```\npip install git+https://github.com/RickBarretto/toCase.git\n```\n\n## Why I must to use it?\n+ toCase was made to make easy your life with case converting\n+ I was made in python, so, if you want, you can copy the [toCase.py](https://github.com/RickBarretto/toCase/blob/main/src/ToCase.py) and use in your project. **It\'s free to use, look the [MIT LICENSE](LICENSE).**\n\n## Glossary:\n- [What is it?](#what-is-it)\n- [Install](#install)\n- [Why I must to use it?](#why-i-must-to-use-it)\n- [Glossary:](#glossary)\n- [Installing](#installing)\n- [Examples:](#examples)\n  - [Importing:](#importing)\n  - [Convert to Camel Case:](#convert-to-camel-case)\n  - [Convert to Snake Case:](#convert-to-snake-case)\n  - [Convert to Kebab Case:](#convert-to-kebab-case)\n  - [Convert to Pascal Case:](#convert-to-pascal-case)\n  - [Convert to Sentence:](#convert-to-sentence)\n- [Read The Docs!](#read-the-docs)\n\n## Installing\n\n```bash\n$ poetry add rb_tocase\n\nor\n\n$ pip install rb_tocase\n```\n\n## Examples: \n\n### Importing:\n```py\n>>> from rb_tocase import Case\n>>> # or\n>>> from rb_tocase import *\n>>> # see the examples below\n```\n\n### Convert to Camel Case:\n```py\n>>> Case.to_camel("Changing to CaMel CASE")   # From String Sentence\n\'changingToCamelCase\'\n>>> Case.to_camel("Changing-to-camel-case")   # From Kebab Case\n\'changingToCamelCase\'\n>>> Case.to_camel("Changing_to_CAMEL_CASE")   # From Snake Case\n\'changingToCamelCase\'\n>>> Case.to_camel(" ChangingToCamelCase  ")   # From Pascal Case\n\'changingToCamelCase\'\n```\n+ [See more](DOC.md#casetocamelstring-str-case1-str--lower)\n\n### Convert to Snake Case:\n```py\n>>> Case.to_snake(" ChanginToSnakeCase ")     # From Pascal Case\n\'changin_to_snake_case\'\n>>> Case.to_snake(" Changin To Snake Case ")  # From String\n\'changin_to_snake_case\'\n>>> Case.to_snake(" Changin-To-Snake-Case ")  # From Kebab\n\'changin_to_snake_case\'\n>>> Case.to_snake(" changinToSnakeCase ")     # From Camel\n\'changin_to_snake_case\'\n```\n[See more](DOC.md#casetosnakestring-str-case-str--lower-case1-str--lower)\n\n### Convert to Kebab Case:\n```py\n>>> Case.to_kebab("Changing to Kebab")    # From String\n\'changing-to-kebab\'\n>>> Case.to_kebab("ChangingToKebab")      # From Pascal Case\n\'changing-to-kebab\'\n>>> Case.to_kebab("changingToKebab")      # From Camel Case\n\'changing-to-kebab\'\n>>> Case.to_kebab("changing_to_kebab")    # From Snake Case\n\'changing-to-kebab\'\n```\n\n[See more](DOC.md#casetokebabstring-str-case-str--lower-case1-str--lower)\n\n### Convert to Pascal Case:\n```py\n>>> Case.to_pascal("Changing to Pascal")  # From String\n\'ChangingToPascal\'\n>>> Case.to_pascal("Changing-to-Pascal")  # From Kebab\n\'ChangingToPascal\'\n>>> Case.to_pascal("Changing_to_Pascal")  # From Snake\n\'ChangingToPascal\'\n>>> Case.to_pascal("ChangingtoPascal")    # From Pascal\n\'ChangingtoPascal\'\n>>> Case.to_pascal("changingToPascal") # From Camel\n\'ChangingToPascal\'\n```\n+ [See more](DOC.md#casetopascalstring-str-case1-str--title)\n\n### Convert to Sentence:\n```py\n>>> Case.to_sentence("ItsAPascalCase")\n\'its a pascal case\'\n>>> Case.to_sentence("itsACamelCase")\n\'its a camel case\'\n>>> Case.to_sentence("Its-A-Kebab-Case")\n\'its a snake case\'\n>>> Case.to_sentence("Its_a_snake_case")\n\'its a snake case\'\n```\n+ [See more](DOC.md#casetosentencestring-str-case-str--lower-case1-str--lower)\n\n\n## Read The Docs!\n[Documentation](DOC.md)',
    'author': 'RickBarretto',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/RickBarretto/toCase',
    'packages': packages,
    'package_data': package_data,
}


setup(**setup_kwargs)
